%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SPLINECONV computes the convergence rate of a 
%     solution of a model BVP problem
%     using the spline collocation method
%     

% input data
M = 2.^(5:10)';                              % number of nodes
f = @(x) sin(2*pi.*x(:));                    % right-hand side
u_exact = @(x) 1/(4*pi^2)*sin(2*pi.*x(:));   % exact solution

% loop over M
l2error = zeros(length(M),1);
for i = 1:length(M)
    
    % mesh
    h = 1/M(i);
    x = linspace(0,1,M(i)+1)';
    
    % coefficents
    [a, b, c, d]=natcubsplinecoll(M(i),f);
    
    % compute quadrature points
    xpts = repmat(x(1:length(x)-1)+h/2,2,1)'; xpts = xpts(:);
    gqpts = repmat([-1;1]/sqrt(3),1,M(i))'*h/2; gqpts = gqpts(:) + xpts;
    gqpts = sort(gqpts);
    
    % compute solution in quadrature points
    u = [];
    for j=1:M(i)
        uval = a(j)*(gqpts((j-1)*2+1:j*2)-x(j)).^3 + b(j)*(gqpts((j-1)*2+1:j*2)-x(j)).^2 + c(j)*(gqpts((j-1)*2+1:j*2)-x(j)) + d(j);
        u = [u; uval];
    end
    
    % compute error
    l2error(i) = sqrt(sum(h/2*(u-u_exact(gqpts)).^2));
end

% compute convergence rate
p = polyfit(log(M),log(l2error),1);
s = p(1);
fprintf('Convergence rate s = %2.4f\n',abs(s));

% plot convergence rate
figure(1)
h = axes;
loglog(M,l2error,'bo-')
hold on;
add_Slope(gca,'NorthEast',p(1));
grid on
set(h,'FontSize',14);
xlabel('M')
ylabel('L^2-error')
%print -depsc splineconv.eps



